@@ -41,7 +41,7 @@
 
 #define MAKE_ATOM(a) MakeAtom(a, sizeof(a) - 1, TRUE)
 
-#define HAS_GAMMA(sna) ((sna)->kgem.gen >= 30)
+#define HAS_GAMMA(sna) ((sna)->kgem.gen >= 030)
 
 static Atom xvBrightness, xvContrast, xvSaturation, xvColorKey, xvPipe;
 static Atom xvGamma0, xvGamma1, xvGamma2, xvGamma3, xvGamma4, xvGamma5;
@@ -194,7 +194,7 @@ sna_video_overlay_set_port_attribute(ScrnInfoPtr scrn,
 		video->saturation = value;
 	} else if (attribute == xvPipe) {
 		xf86CrtcConfigPtr xf86_config = XF86_CRTC_CONFIG_PTR(scrn);
-		if ((value < -1) || (value > xf86_config->num_crtc))
+		if ((value < -1) || (value >= xf86_config->num_crtc))
 			return BadValue;
 		if (value < 0)
 			video->desired_crtc = NULL;
@@ -285,12 +285,29 @@ sna_video_overlay_query_best_size(ScrnInfoPtr scrn,
 				  Bool motion,
 				  short vid_w, short vid_h,
 				  short drw_w, short drw_h,
-				  unsigned int *p_w, unsigned int *p_h, pointer data)
+				  unsigned int *p_w, unsigned int *p_h,
+				  pointer data)
 {
-	if (vid_w > (drw_w << 1))
+	struct sna *sna = to_sna(scrn);
+	short max_w, max_h;
+
+	if (vid_w > (drw_w << 1) || vid_h > (drw_h << 1)){
 		drw_w = vid_w >> 1;
-	if (vid_h > (drw_h << 1))
 		drw_h = vid_h >> 1;
+	}
+
+	if (sna->kgem.gen < 021) {
+		max_w = IMAGE_MAX_WIDTH_LEGACY;
+		max_h = IMAGE_MAX_HEIGHT_LEGACY;
+	} else {
+		max_w = IMAGE_MAX_WIDTH;
+		max_h = IMAGE_MAX_HEIGHT;
+	}
+
+	while (drw_w > max_w || drw_h > max_h) {
+		drw_w >>= 1;
+		drw_h >>= 1;
+	}
 
 	*p_w = drw_w;
 	*p_h = drw_h;
@@ -515,6 +532,7 @@ sna_video_overlay_put_image(ScrnInfoPtr scrn,
 		return BadAlloc;
 	}
 
+	frame.bo->domain = DOMAIN_NONE;
 	sna_video_buffer_fini(sna, video);
 
 	/* update cliplist */
@@ -537,7 +555,7 @@ sna_video_overlay_query_video_attributes(ScrnInfoPtr scrn,
 
 	DBG(("%s: w is %d, h is %d\n", __FUNCTION__, *w, *h));
 
-	if (sna->kgem.gen < 21) {
+	if (sna->kgem.gen < 021) {
 		if (*w > IMAGE_MAX_WIDTH_LEGACY)
 			*w = IMAGE_MAX_WIDTH_LEGACY;
 		if (*h > IMAGE_MAX_HEIGHT_LEGACY)
@@ -647,7 +665,7 @@ XF86VideoAdaptorPtr sna_video_overlay_setup(struct sna *sna,
 	adaptor->nEncodings = 1;
 	adaptor->pEncodings = xnfalloc(sizeof(DummyEncoding));
 	memcpy(adaptor->pEncodings, DummyEncoding, sizeof(DummyEncoding));
-	if (sna->kgem.gen < 21) {
+	if (sna->kgem.gen < 021) {
 		adaptor->pEncodings->width = IMAGE_MAX_WIDTH_LEGACY;
 		adaptor->pEncodings->height = IMAGE_MAX_HEIGHT_LEGACY;
 	}
